import { jsx as _jsx } from "react/jsx-runtime";
import _extends from "@babel/runtime/helpers/esm/extends";
import { useContext, useEffect, useRef, useState } from 'react';
import SUIContext from '@s-ui/react-context'; // used to store the last definition of ClientPage component so it can be reused

var latestClientPage = null;

var getInitialPropsFromWindow = function getInitialPropsFromWindow() {
  // if no window initial props, then do nothing
  if (typeof window.__INITIAL_PROPS__ === 'undefined') return; // make a copy of the content safely

  var windowInitialProps = _extends({}, window.__INITIAL_PROPS__); // remove the variable of the window


  window.__INITIAL_PROPS__ = undefined; // return retrieved props from window

  return windowInitialProps;
}; // extract needed info from props for routeInfo object


var createRouteInfoFromProps = function createRouteInfoFromProps(_ref) {
  var location = _ref.location,
      params = _ref.params,
      routes = _ref.routes;
  return {
    location: location,
    params: params,
    routes: routes
  };
}; // This HoC creates the PageComponent prepared for the Client Side Rendering
// It renders a placeholder, if any specified as `renderLoading` on the PageComponent.
// Also, executes the getInitialProps to retrieve the initialProps and then
// renders the component with the initialProps, hiding the placeholder.
//
// Additionally, if `keepMounted` on the PageComponent is true, it keeps the
// component alive by running getInitialProps again when a route change is done
// and the very same component is matched by the router, so PageComponent just
// gets props updated. Also, since PageComponent keeps mounted it will receive
// an `isLoading` prop while getInitialProps is in progress.


export default (function (Page) {
  var _latestClientPage, _latestClientPage$Pag;

  // gather window initial props for this Page, if present
  var windowInitialProps = getInitialPropsFromWindow(); // define Page wrapper component

  var ClientPage = function ClientPage(props) {
    var initialPropsFromWindowRef = useRef(windowInitialProps); // used to know if initialProps has been requested at least once

    var requestedInitialPropsOnceRef = useRef(!!windowInitialProps); // create routeInfo object from current props which are updated

    var routeInfo = createRouteInfoFromProps(props); // consume sui context from the context provider

    var suiContext = useContext(SUIContext); // pathName from context is outdated, so we update it from routeInfo

    var context = _extends({}, suiContext, {
      pathName: routeInfo.location.pathname
    });

    var _useState = useState(function () {
      return {
        initialProps: initialPropsFromWindowRef.current || {},
        isLoading: !initialPropsFromWindowRef.current
      };
    }),
        _useState$ = _useState[0],
        initialProps = _useState$.initialProps,
        isLoading = _useState$.isLoading,
        setState = _useState[1];

    useEffect(function () {
      // check if got initial props from window, because then there's no need
      // to request them again from client
      if (initialPropsFromWindowRef.current) {
        initialPropsFromWindowRef.current = undefined;
      } else {
        // only update state if already request initial props
        if (requestedInitialPropsOnceRef.current) {
          setState({
            initialProps: initialProps,
            isLoading: true
          });
        }

        Page.getInitialProps({
          context: context,
          routeInfo: routeInfo
        }).then(function (initialProps) {
          setState({
            initialProps: initialProps,
            isLoading: false
          });
        })["catch"](function (error) {
          setState({
            initialProps: {
              error: error
            },
            isLoading: false
          });
        })["finally"](function () {
          if (requestedInitialPropsOnceRef.current) return;
          requestedInitialPropsOnceRef.current = true;
        });
      }
    }, [routeInfo.location]); // eslint-disable-line react-hooks/exhaustive-deps

    var renderPage = function renderPage() {
      return /*#__PURE__*/_jsx(Page, _extends({}, initialProps, props, {
        isLoading: isLoading
      }));
    }; // if the page has a `keepMounted` property and already requested
    // initialProps once, just keep rendering the page


    if (Page.keepMounted && requestedInitialPropsOnceRef.current) {
      return renderPage();
    }

    var renderLoading = function renderLoading() {
      // check if the page has a `renderLoading` method, if not, just render nothing
      return Page.renderLoading ? Page.renderLoading({
        context: context,
        routeInfo: routeInfo
      }) : null;
    };

    return isLoading ? renderLoading() : renderPage();
  }; // if `keepMounted` property is found and the component is the same one,
  // we just reuse it instead of returning a new one


  if (Page.keepMounted && Page.displayName === ((_latestClientPage = latestClientPage) == null ? void 0 : (_latestClientPage$Pag = _latestClientPage.Page) == null ? void 0 : _latestClientPage$Pag.displayName)) {
    return latestClientPage;
  } // save original page component


  ClientPage.Page = Page; // add ClientPage to name of the component

  ClientPage.displayName = "ClientPage(" + Page.displayName + ")"; // save ClientPage to latestClientPage

  latestClientPage = ClientPage; // return the page

  return ClientPage;
});